/**
 *	@file	ya_basic_string.h
 *	@brief	std::string ̌֕i.
 *	@author	tenk* (Masashi Kitamura)
 *	@date	2008-2010
 *	@note
 *	- c_str() '\0'̕t́Ac_str()ɍsB
 *	  resize()ł͗]1mۂāAȂׂ c_str()resize()
 *	  Ȃ悤ɂĂ.
 *	- ɗ́Axmۂ̓TCYӂȂg܂킷.
 *	  mۃk߂ꍇ́AOswapȂȂȂ.
 *	- ꉞAO𓊂̂ftHgAOstd::stringg߁A
 *	  {]|. UNUSE_THROW `Ăassert`FbNɕύX.
 *	- G[̗Oassertvc8^Ă邪AK̂Œ.
 *	- XAO𓊂ȂԂł̗plĂ邽߁A͈͊Ow
 *	  Kɒ킹..OꂵĂȂ̂Œ.
 *	- {IɃRs[TRAITS::copyōsĂ邪
 *	  Ce[^ɂ͂̏ꍇ std::copy gĂ.
 *	  }`oCgn̏ftHchar_traitsdl̂܂܂ł͋złȂ
 *		̂ŋCɂKvȂ.
 *	- Public Domain Software
 */
#ifndef YA_STRING_H
#define YA_STRING_H

#pragma once

#include <cstddef>
#include <cstring>
#include <iterator>
#include <algorithm>
#include <vector>
#ifndef assert
#include <cassert>
#endif
#ifndef UNUSE_THROW
#include <stdexcept>
#endif

#if defined __WATCOMC__ || defined __DMC__
#define YA_STRING_NO_MEMBER_TEMPLATES
#endif


// ============================================================================
// AP[^
// ============================================================================

#ifndef YA_ALLOCATOR_INCLUDED
#define YA_ALLOCATOR_INCLUDED
template< typename T >
class ya_allocator {
public:
	typedef T				value_type;
	typedef T*				pointer;
	typedef const T*		const_pointer;
	typedef T&				reference;
	typedef const T&		const_reference;
	//typedef std::size_t	size_type;
	typedef unsigned		size_type;				// 64rbgCPUTCY32rbgȓƂ.
	typedef std::ptrdiff_t	difference_type;
	template <class U> struct rebind { typedef class ya_allocator<U> other; };

	ya_allocator() {}
	ya_allocator(const ya_allocator&) {}
  #ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
	template<class U> ya_allocator(const ya_allocator<U>&) {}
  #endif
	~ya_allocator() throw() {}

	static pointer			address(reference		x) {return &x;}
	static const_pointer	address(const_reference x) {return &x;}

	static pointer allocate(size_type n)			  {return (pointer)::operator new(sizeof(T)*n);}
	static pointer allocate(size_type n,const void *) {return allocate(n);}

	static void deallocate(pointer p)				{::operator delete(p);}
	static void deallocate(pointer p, size_type)	{deallocate(p);}

	static void construct(pointer p, const T& val)	{new((void*)p) T(val);}
	static void construct(pointer p)				{new((void*)p) T;}
	static void destroy(pointer p)					{p->~T();}

	static size_type max_size() throw() {return ~0U; }

  #ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
	template<class U> bool operator ==(const ya_allocator<U>& r) const {return true;}
	template<class U> bool operator !=(const ya_allocator<U>& r) const {return !(*this == r);}
  #else
	bool operator ==(const ya_allocator& r) const {return true;}
	bool operator !=(const ya_allocator& r) const {return !(*this == r);}
  #endif
};
#endif	// YA_ALLOCATOR_INCLUDED



// ============================================================================
// NX.
// ============================================================================

#ifndef YA_CHAR_TRAITS_DEFINED
#define YA_CHAR_TRAITS_DEFINED

/// basic_v_stringňɊւ traits. ق std::char_traitsƓl.
template<typename charT>
class ya_char_traits {
	//typedef ucharT				char_cmp_type;
	typedef charT					char_cmp_type;

public:
	typedef charT		 			char_type;
	typedef int 					int_type;
	typedef unsigned				pos_type;
	typedef int 					off_type;
	typedef unsigned				state_type;
	typedef typename std::size_t	size_type;

	static int_type 		eof() { return -1; }
	static int_type 		not_eof( const int_type& c ) { return c >= 0 ? c : 0; }
	static bool 			eq( const char_type& a, const char_type& b ) {return (a == b); }

	static bool 			eq_int_type( const int_type& a, const int_type& b ) { return (a == b); }
	static bool 			lt( const char_type& a, const char_type& b ) { return char_cmp_type(a) < char_cmp_type(b); }
	static void 			assign( char_type& d, const char_type& s ) { d = s; }

	static char_type 		to_char_type( const int_type& c ) { return char_type(c); }
	static int_type 		to_int_type( const char_type& c ) { return char_cmp_type(c); }

	static char_type*		assign( char_type* dst, std::size_t n, const char_type& c );
	static char_type*		copy( char_type* dst, const char_type* s, std::size_t n );
	static char_type*		move( char_type* dst, const char_type* s, std::size_t n );
	static size_type 		length( const char_type *str );
	static int		 		compare( const char_type* a, const char_type* b, std::size_t n );
	static const char_type* find( const char_type* str, std::size_t n, const char_type& k );
};



template<typename charT>
charT* ya_char_traits<charT>::assign(charT* dst, std::size_t n, const charT& c)
{
	assert(dst != NULL);
	if (n) {
		charT *d = dst;
		charT *e = d + n;
		do {
			*d++ = c;
		} while (d != e);
	}
	return dst;
}


template<typename charT>
charT* ya_char_traits<charT>::copy(charT* dst, const charT* s, std::size_t n)
{
	assert(dst != NULL);
	assert(s   != NULL);
	if (n) {
		charT *d = dst;
		charT *e = d + n;
		do {
			*d++ = *s++;
		} while (d != e);
	}
	return dst;
}


template<typename charT>
charT* ya_char_traits<charT>::move(charT* dst, const charT* s, std::size_t n)
{
	assert(dst != NULL && s != NULL);
	if ((n != 0) & (dst != 0)) {
		charT *d = dst;
		if (d < s) {
			charT *e = d + n;
			do {
				*d++ = *s++;
			} while (d != e);
		} else if (d > s) {
			d += n;
			s += n;
			do {
				*--d = *--s;
			} while (d != dst);
		}
	}
	return dst;
}



template<typename charT>
std::size_t ya_char_traits<charT>::length(const charT *str)
{
	size_type 	l  = 0;
	const charT *s = str;
	if (s) {
		while (*s != charT())
			++s;
		l = s - str;
	}
	return l;
}



template<typename charT>
int		ya_char_traits<charT>::compare(
		const charT* 	a,
		const charT* 	b,
		std::size_t 	n
){
	assert(a != NULL && b != NULL);
	const char_cmp_type *l = (const char_cmp_type *)a;
	const char_cmp_type *r = (const char_cmp_type *)b;
	const char_cmp_type *e = l + n;
	int c = 0;
	while (l != e && ((c = *l - *r++) == 0) && (*l != charT()))
		++l;
	return c;
}



template<typename charT>
const charT* ya_char_traits<charT>::find(const charT* src, std::size_t n, const charT& key)
{
	if (n && src) {
		const char_cmp_type *s = (const char_cmp_type *)src;
		const char_cmp_type *e = s + n;
		const char_cmp_type  k = char_cmp_type(key);
		do {
			if (*s == k)
				return reinterpret_cast<const charT *>(s);
			++s;
		} while (s != e);
	}
	return NULL;
}



// ---------------------------------------------------------
// char p̎.
#if 1
/// charpłya_char_traits.
template<>
class ya_char_traits<char> {
	typedef unsigned char	char_cmp_type;

public:
	typedef char	   		char_type;
	typedef int 			int_type;
	typedef unsigned		pos_type;
	typedef int 			off_type;
	typedef unsigned		state_type;
	typedef std::size_t		size_type;

	static int_type 		eof() 								{ return -1; }
	static int_type 		not_eof( int_type c ) 				{ return c >= 0 ? c : 0; }
	static bool 			eq( char_type a, char_type b ) 		{ return (a == b); }
	static bool 			eq_int_type(int_type a, int_type b) { return (a == b); }
	static bool 			lt( char_type a, char_type b ) 		{ return char_cmp_type(a) < char_cmp_type(b); }
	static void 			assign( char_type& d, char_type s ) { d = s; }
	static char_type 		to_char_type( int_type c ) 			{ return char_type(c) ; }
	static int_type 		to_int_type( char_type c ) 			{ return char_cmp_type(c); }

	static size_type 		length( const char_type *str )							  { return ::strlen(str); }
	static char_type*		assign( char_type* d, size_type n, char_type c )		  { return (char_type*)::memset (d,c,n);}
	static char_type*		copy( char_type* d, const char_type* s, size_type n ) 	  { return (char_type*)::memcpy (d,s,n);}
	static char_type*		move( char_type* d, const char_type* s, size_type n ) 	  { return (char_type*)::memmove(d,s,n);}
	static int		 		compare(const char_type* a,const char_type* b,size_type n){ return ::memcmp(a,b,n); }
	static const char_type* find( const char_type* s, size_type n, char_type key )	  { return (char_type*)::memchr(s,key,n);}
};
#endif

#endif		// YA_CHAR_TRAITS_DEFINED



// ============================================================================
// NX.
// ============================================================================

template< typename C, class T=ya_char_traits<C>, class A=ya_allocator<C> >
class ya_basic_string : protected A {
	typedef ya_basic_string	this_type;
	enum {	ALIGN_BIT		= 4 };
public:
	// ^
	typedef C										char_type;
	typedef typename A::size_type					size_type;
	typedef typename A::difference_type				difference_type;
	typedef typename A::pointer						pointer;
	typedef typename A::const_pointer				const_pointer;
	typedef typename A::reference					reference;
	typedef typename A::const_reference				const_reference;
	typedef pointer									iterator;
	typedef const_pointer							const_iterator;
	typedef std::reverse_iterator<iterator> 		reverse_iterator;
	typedef std::reverse_iterator<const_iterator>	const_reverse_iterator;
	typedef A										allocator_type;
	static const std::size_t						npos;

	// RXgN^fXgN^
	ya_basic_string() : ptr_(0), size_(0), capa_(0) {}									///< ftHgRXgN^
	explicit ya_basic_string(std::size_t n) :ptr_(0),size_(0),capa_(0) {resize(n,C());}	///< n̋
	ya_basic_string(std::size_t n,char_type c):ptr_(0),size_(0),capa_(0) {resize(n,c);}	///< ncŏ
	ya_basic_string(const char_type* cstr);												///< s̏I܂
	ya_basic_string(const char_type* cary, std::size_t n);								///< sn
	ya_basic_string(const ya_basic_string& str);										///< Rs[RXgN^
	ya_basic_string(const ya_basic_string& str, std::size_t pos, std::size_t n=npos);	///< strposڂn
	// ~ya_basic_string() {}															///< fXgN^

	// TCY
	bool 			empty()    const { return size_ == 0; }								///< Ȃtrue
	std::size_t		size()	   const { return size_; }									///< Ԃ
	std::size_t		length()   const { return size_; }									///< Ԃ
	std::size_t		max_size() const { return capa_; }									///< mۉ\ȍő啶Ԃ
	void 			resize(std::size_t n, char_type c=C());								///< nɕςAcŖ߂.
	std::size_t		capacity() const { return capa_; }									///< ݂̃obt@TCY.
	void 			reserve(std::size_t l);												///< obt@TCYύX.

	// ANZX
	const char_type* 	c_str() const;													///< '\0'I[̃AhXԂ.
	char_type* 			data() 			  { return ptr_; }								///< f[^{̂̂܂ܕԂ.
	const char_type* 	data() const	  { return ptr_; }								///< f[^{̂̂܂ܕԂ.
	const_reference operator[](std::size_t n) const {assert(n<size_);return ptr_[n];} 	///< nڂ̕Ԃ.
	reference		operator[](std::size_t n)		{assert(n<size_);return ptr_[n];}	///< nڂ̕Ԃ.
	const_reference at(std::size_t n) const {if (n>=size_) outOfRng();return ptr_[n];}	///< nڂ̕Ԃ. ͈͊OȂO.
	reference 		at(std::size_t n) 		{if (n>=size_) outOfRng();return ptr_[n];}	///< nڂ̕Ԃ. ͈͊OȂO.

	// Ce[^
	iterator 			   begin() 		  { return ptr_; }								///< 擪ւ̃Ce[^Ԃ
	const_iterator 		   begin() const  { return ptr_; }								///< 擪ւ̃Ce[^Ԃ
	iterator 			   end()		  { return ptr_+size_; }						///< I[(̎)wCe[^Ԃ
	const_iterator 		   end()   const  { return ptr_+size_; }						///< I[(̎)wCe[^Ԃ
	reverse_iterator	   rbegin() 	  { return reverse_iterator( end() );}			///< 擪wtCe[^Ԃ
	const_reverse_iterator rbegin() const { return const_reverse_iterator(end());}		///< 擪wtCe[^Ԃ
	reverse_iterator	   rend()		  { return reverse_iterator( begin() ); }		///< I[(̎)wtCe[^Ԃ
	const_reverse_iterator rend()	const { return const_reverse_iterator(begin());}	///< I[(̎)wtCe[^Ԃ

	const_iterator 		   cbegin() const { return ptr_; }								///< 擪ւ̃Ce[^Ԃ
	const_iterator 		   cend()	const { return ptr_+size_; }						///< I[(̎)wCe[^Ԃ
	const_reverse_iterator crbegin()const { return const_reverse_iterator(end());}		///< 擪wtCe[^Ԃ
	const_reverse_iterator crend()	const { return const_reverse_iterator(begin());}	///< I[(̎)wtCe[^Ԃ

	// r
	int compare(const ya_basic_string& str) const;
	int compare(std::size_t pos1, std::size_t n1, const ya_basic_string& str) const;
	int compare(std::size_t pos1, std::size_t n1, const ya_basic_string& str, std::size_t pos2, std::size_t n2) const;
	int compare(const char_type* s) const;
	int compare(const char_type* s, std::size_t n) const;
	int compare(std::size_t pos1, std::size_t n1, const char_type* cstr) const;
	int compare(std::size_t pos1, std::size_t n1, const char_type* cary, std::size_t n2) const;

	// 
	ya_basic_string& operator=(const ya_basic_string& str);
	ya_basic_string& operator=(const char_type* cstr);
	ya_basic_string& operator=(char_type c);

	ya_basic_string& assign(const ya_basic_string& str);
	ya_basic_string& assign(const ya_basic_string& str, std::size_t pos, std::size_t n=npos);
	ya_basic_string& assign(const char_type* cstr);
	ya_basic_string& assign(const char_type* cary, std::size_t n);
	ya_basic_string& assign(std::size_t n, char_type c);

	// ǉ
	ya_basic_string& operator+=(const ya_basic_string& str);
	ya_basic_string& operator+=(const char_type* cstr);
	ya_basic_string& operator+=(char_type c);

	ya_basic_string& append(const ya_basic_string& str);
	ya_basic_string& append(const ya_basic_string& str, std::size_t pos, std::size_t n=npos);
	ya_basic_string& append(const char_type* cstr);
	ya_basic_string& append(const char_type* cary, std::size_t n);
	ya_basic_string& append(std::size_t n, char_type c);

	// }
	ya_basic_string& insert(std::size_t pos1, const ya_basic_string& str, std::size_t pos2 = 0, std::size_t n = npos);
	ya_basic_string& insert(std::size_t pos, const char_type* cstr, std::size_t n);
	ya_basic_string& insert(std::size_t pos, const char_type* cary);
	ya_basic_string& insert(std::size_t pos, std::size_t n, char_type c);
	iterator  		 insert(iterator p, char_type c);
	iterator  		 insert(iterator p, std::size_t n, char_type c);

	// 폜
	ya_basic_string& erase() { destruct(ptr_, size_); size_ = 0; return *this; }
	ya_basic_string& erase(std::size_t pos, std::size_t n = npos);
	iterator 		 erase(iterator pos);
	iterator 		 erase(iterator first, iterator last);

	// u
	//ya_basic_string& replace(std::size_t pos1, std::size_t n1, const ya_basic_string& str2);
	ya_basic_string& replace(std::size_t pos1, std::size_t n1, const ya_basic_string& str2, std::size_t pos2 = 0, std::size_t n2 = npos);
	ya_basic_string& replace(std::size_t pos1, std::size_t n1, const char_type* cstr2);
	ya_basic_string& replace(std::size_t pos1, std::size_t n1, const char_type* cary2, std::size_t n2);
	ya_basic_string& replace(std::size_t pos1, std::size_t n1, std::size_t n2, char_type c2);
	ya_basic_string& replace(iterator fi, iterator li, const ya_basic_string& str);
	ya_basic_string& replace(iterator fi, iterator li, const char_type* cstr);
	ya_basic_string& replace(iterator fi, iterator li, const char_type* cary, std::size_t n);
	ya_basic_string& replace(iterator fi, iterator li, std::size_t n, char_type c);


	// ()̏
	std::size_t find(char_type c2) const;
	std::size_t find(char_type c2, std::size_t pos1) const;
	std::size_t find(const ya_basic_string& str2) const;
	std::size_t find(const ya_basic_string& str2, std::size_t pos1) const;
	std::size_t find(const char_type* cstr2) const;
	std::size_t find(const char_type* cstr2, std::size_t pos1) const;
	std::size_t find(const char_type* cary2, std::size_t pos1, std::size_t n2) const;

	// ()̋t
	std::size_t rfind(char_type c) const;
	std::size_t rfind(char_type c, std::size_t pos) const;
	std::size_t rfind(const ya_basic_string& str) const;
	std::size_t rfind(const ya_basic_string& str, std::size_t pos) const;
	std::size_t rfind(const char_type* cstr) const;
	std::size_t rfind(const char_type* cstr, std::size_t pos) const;
	std::size_t rfind(const char_type* cstr, std::size_t pos, std::size_t n) const;

	// w蕶̏
	std::size_t find_first_of(char_type c) const;
	std::size_t find_first_of(char_type c, std::size_t pos) const;
	std::size_t find_first_of(const ya_basic_string& str) const;
	std::size_t find_first_of(const ya_basic_string& str, std::size_t pos) const;
	std::size_t find_first_of(const char_type *cstr) const;
	std::size_t find_first_of(const char_type *cstr, std::size_t pos) const;
	std::size_t find_first_of(const char_type *cstr, std::size_t pos, std::size_t n) const;

	// w蕶̋t
	std::size_t find_last_of(char_type c) const;
	std::size_t find_last_of(char_type c, std::size_t pos) const;
	std::size_t find_last_of(const ya_basic_string& str) const;
	std::size_t find_last_of(const ya_basic_string& str, std::size_t pos) const;
	std::size_t find_last_of(const char_type *cstr) const;
	std::size_t find_last_of(const char_type *cstr, std::size_t pos) const;
	std::size_t find_last_of(const char_type *cstr, std::size_t pos, std::size_t n) const;

	// w蕶ȊȌ
	std::size_t find_first_not_of(char_type c) const;
	std::size_t find_first_not_of(char_type c, std::size_t pos) const;
	std::size_t find_first_not_of(const ya_basic_string& str) const;
	std::size_t find_first_not_of(const ya_basic_string& str, std::size_t pos) const;
	std::size_t find_first_not_of(const char_type *cstr) const;
	std::size_t find_first_not_of(const char_type *cstr, std::size_t pos) const;
	std::size_t find_first_not_of(const char_type *cstr, std::size_t pos, std::size_t n) const;

	// w蕶ȊŐt
	std::size_t find_last_not_of(char_type c) const;
	std::size_t find_last_not_of(char_type c, std::size_t pos) const;
	std::size_t find_last_not_of(const ya_basic_string& str) const;
	std::size_t find_last_not_of(const ya_basic_string& str, std::size_t pos) const;
	std::size_t find_last_not_of(const char_type *cstr) const;
	std::size_t find_last_not_of(const char_type *cstr, std::size_t pos) const;
	std::size_t find_last_not_of(const char_type *cstr, std::size_t pos, std::size_t n) const;

	// sposڂnRs[
	std::size_t copy(char_type* cary, std::size_t n, std::size_t pos = 0) const;

	// sƂ̓ꊷ
	void swap(ya_basic_string& r) { std::swap(ptr_,r.ptr_); std::swap(size_,r.size_); std::swap(capa_,r.capa_); }

	// posڂnAVKĕԂ.
	ya_basic_string substr(std::size_t pos = 0, std::size_t n = npos) const;

	// // AP[^Ԃ.
	A&			get_allocator() const { return *(A*)this; }

	// o[ev[g֌W.
  #ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
	template<class InpIte> ya_basic_string(InpIte first, InpIte last);					///< Ce[^ŏ
	template<class InpIte> ya_basic_string& assign(InpIte first, InpIte last);
	template<class InpIte> ya_basic_string& append(InpIte first, InpIte last);
	template<typename InpIte> iterator insert(iterator p, InpIte first, InpIte last);
	template<class InpIte> ya_basic_string& replace(iterator fi1, iterator li1, InpIte fi2, InpIte li2);
  #else									// o[ev[gT|[g̏ꍇ.
	ya_basic_string(const C* first, const C* last);										///< Ce[^ŏ
	ya_basic_string& assign(const C* first, const C* last);
	ya_basic_string& append(const C* first, const C* last);
	iterator		 insert(iterator p, const C* first, const C* last);
	ya_basic_string& replace(iterator fi1, iterator li1, const C* fi2, const C* li2);
  #endif

private:
	pointer		   	ptr(std::size_t pos) 		{ return ptr_ + pos; }
	const_pointer  	ptr(std::size_t pos) const	{ return ptr_ + pos; }
	iterator	   	ite(std::size_t pos) 		{ return ptr_ + pos; }
	const_iterator 	ite(std::size_t pos) const	{ return ptr_ + pos; }

	void			check_pos(std::size_t& pos) const;
	void			check_pos_len(std::size_t& pos, std::size_t& len) const;
	void			check_pos(iterator& pos) const;
	void			check_range(iterator& first, iterator& last) const;

	static std::size_t	find_1(const char_type* cary1, std::size_t ofs1, std::size_t n1, char_type c2);
	static std::size_t	find_str(const char_type* cary1, std::size_t ofs1, std::size_t n1, const char_type* cary2, std::size_t n2);
	static std::size_t	rfind_1(const char_type* cary1, std::size_t n1, char_type c2);
	static std::size_t	rfind_str(const char_type* cary1, std::size_t n1, const char_type* cary2, std::size_t n2);
	static std::size_t	fnd_first_of(const char_type* cary1, std::size_t ofs1, std::size_t n1, const char_type* cary2, std::size_t n2);
	static std::size_t	fnd_last_of(const char_type* cary1, std::size_t n1, const char_type* cary2, std::size_t n2);
	static std::size_t	fnd_first_not_of(const char_type* cary1, std::size_t ofs1, std::size_t n1, char_type c2);
	static std::size_t	fnd_first_not_of(const char_type* cary1, std::size_t ofs1, std::size_t n1, const char_type* cary2, std::size_t n2);
	static std::size_t	fnd_last_not_of(const char_type* cary1, std::size_t n1, char_type c2);
	static std::size_t	fnd_last_not_of(const char_type* cary1, std::size_t n1, const char_type* cary2, std::size_t n2);

  #ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
	template<typename InpIte> inline
	void	copy_construct(C* d, InpIte s, InpIte e);
  #else									// o[ev[gT|[g̏ꍇ.
	void	copy_construct(C* d, const C* s, const C* e);
  #endif

	void	set_construct(C* a, const C& t, std::size_t sz);
	void	destruct(C* a, std::size_t sz);

  #ifdef UNUSE_THROW
	void outOfRng() const { assert(0 && "out of range in ya_basic_string"); }
  #else
	void outOfRng() const { throw std::out_of_range("out of range in ya_basic_string"); }
  #endif

private:
	C*				ptr_;				///< .
	size_type		size_;				///< .
	size_type		capa_;				///< ptr_̗̈TCY.
};



//---------------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------------

template< typename C, class T, class A >
const std::size_t		ya_basic_string<C,T,A>::npos = std::size_t(-1);


// -------------------------------
// private
// -------------------------------

/// AhX dsztŖ߂.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::set_construct(C* a, const C& t, std::size_t sz) {
	if (sz) {
		assert(a != 0);
		C* e = a + sz;
		do {
			A::construct(a, t);
		} while (++a != e);
	}
}



/// [s,e)Rs[RXgN^gāAAhXdփRs[.
/// [private]
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template<typename InpIte> inline
void ya_basic_string<C,T,A>::copy_construct(C* d, InpIte s, InpIte e)
#else
template< typename C, class T, class A >
void ya_basic_string<C,T,A>::copy_construct(C* d, const C* s, const C* e)
#endif
{
	assert(d != 0);
	while (s != e) {
		A::construct(d++, *s);
		++s;
	}
}



/// AhX aszAfXgN^s.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::destruct(C* a, std::size_t sz) {
	// assert(sz > 0);
	if (sz) {
		assert(a != 0);
		C* e = a + sz;
		do {
			A::destroy(a);
		} while (++a != e);
	}
}




// -----------------------------------


/// ʒuwpos͈͊Oǂ`FbNA͈͊OȂstd::out_of_range𓊂.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::check_pos(std::size_t& pos) const {
	if (pos > size_) {
		outOfRng();
		pos = size_;
	}
}


/// ʒu񂪔͈͊OȂO𓊂. ܂AΒ.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::check_pos_len(std::size_t& pos, std::size_t& len) const
{
	std::size_t l = size_;
	if (pos > l) {
		outOfRng();
		pos = l;
	}
	if (len > l - pos) {
		len = l - pos;
	}
}


/// ʒuwpos͈͊Oǂ`FbNA͈͊OȂstd::out_of_range𓊂.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::check_pos(typename ya_basic_string<C,T,A>::iterator& pos) const
{
	if (pos < begin() || end() < pos) {
		outOfRng();
		pos = const_cast< this_type* >(this)->end();
	}
}



/// ͈͎w[first,last)͈͊Oǂ`FbN&.
/// [private]
template< typename C, class T, class A > inline
void ya_basic_string<C,T,A>::check_range(
		typename ya_basic_string<C,T,A>::iterator&	first,
		typename ya_basic_string<C,T,A>::iterator&	last
) const
{
  #ifdef UNUSE_THROW
	if (first < begin()) {
		outOfRng();
		first = const_cast< this_type* >(this)->begin();
	} else if (end() < first) {
		outOfRng();
		first = const_cast< this_type* >(this)->end();
	}
	if (last < begin()) {
		outOfRng();
		last = const_cast< this_type* >(this)->begin();
	} else if (end() < last) {
		outOfRng();
		last = const_cast< this_type* >(this)->end();
	}
  #else
	if (   first > last
		|| first < begin() || end() < first
		|| last  < begin() || end() < last
	) {
		outOfRng();
	}
  #endif
}



// -------------------------------
// TCY֌W.

template< typename C, class T, class A >
void ya_basic_string<C,T,A>::reserve( std::size_t capa ) {
	if (capa > capa_) {
		std::size_t algn_msk	= (1 << ALIGN_BIT) - 1;
		capa	= (capa + algn_msk) & ~algn_msk;
		C*	a = A::allocate( capa );
		assert(a != NULL);
		if (a) {
			if (ptr_) {
				std::uninitialized_copy(ptr_, ptr_+capa, a);
				A::deallocate( ptr_ );
			}
			ptr_ = a;
		}
		capa_ = capa;
	}
}


/** ̒(length)nɕςāA傫ȂcŖ߂.
 */
template< typename C, class T, class A >
void ya_basic_string<C,T,A>::resize( std::size_t sz, C c) {
  	std::size_t	size = size_;
	if (sz < size) {
		destruct(ptr_+sz, size - sz);
	} else if (sz > size) {
		if (sz > capa_) {
			reserve( sz+1 );	// ǂmۂȂ\'\0'̕m.
		}
		set_construct(ptr_+size, c, sz - size);
	}
	size_ = sz;
}



// -------------------------------

/** '\0'I[ւ̃|C^Ԃ.
 *	 const֐ɂĂ邪Aۂɂ͈Öق\0ޏ.
 *	@@ꍇɂĂresize()s.
 */
template< typename C, class T, class A > inline
const C* ya_basic_string<C,T,A>::c_str() const {
	std::size_t		 sz	 = size_;
	if (sz >= capa_)
		const_cast< ya_basic_string* >(this)->reserve( sz + 1 );
	const_cast< ya_basic_string* >(this)->ptr_[sz] = char_type();
	return ptr_;
}



// -------------------------------
// RXgN^.

/** cstrRs[RXgN^.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>::ya_basic_string(const C* cstr)
	: ptr_(0), size_(0), capa_(0)
{
	std::size_t	l = 0;
	if (cstr)
		l = T::length(cstr);
	if (l) {
		resize(l);
		T::copy(ptr_, cstr, l);
	}
}



/** carynRs[RXgN^.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>::ya_basic_string(const C* cary, std::size_t n)
	: ptr_(0), size_(0), capa_(0)
{
	if (n) {
		resize(n);
		T::copy(ptr_, cary, n);
	}
}



/** Rs[RXgN^.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>::ya_basic_string(const ya_basic_string<C,T,A>& str)
	: ptr_(0), size_(0), capa_(0)
{
	std::size_t l = str.size_;
	if (l) {
		resize(l);
		T::copy( ptr_, str.ptr_, l);
	}
}


/** strposڂnRs[RXgN^.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>::ya_basic_string( const ya_basic_string<C,T,A>& str, std::size_t pos, std::size_t n )
	: ptr_(0), size_(0), capa_(0)
{
	str.check_pos_len(pos, n);
	if (n) {
		resize(n);
		T::copy( ptr_, str.ptr(pos), n);
	}
}



/** Ce[^ŏRXgN^.
 */
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template<class InpIte>
inline ya_basic_string<C,T,A>::ya_basic_string(InpIte first2, InpIte last2)
	: ptr_(0), size_(0), capa_(0)
{
	std::size_t	l2 = std::distance(first2, last2);
	if (l2) {
		resize(l2);
		std::copy(first2, last2, ptr_);
	}
}
#else
template< typename C, class T, class A >
ya_basic_string<C,T,A>::ya_basic_string(const C* first2, const C* last2)
	: ptr_(0), size_(0), capa_(0)
{
	std::size_t	l2 = last2 - first2;
	if (l2) {
		resize(l2);
		T::copy(ptr_, first2, l2 );
	}
}
#endif



// -------------------------------
// 

/** str.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::operator=(const ya_basic_string<C,T,A>& str) {
	return assign(str);
}


/** cstr.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::operator=(const C* cstr) {
	return assign(cstr);
}


/** c.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::operator=(C c) {
	resize(1);
	ptr_[0] = c;
	return *this;
}


/** nc.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign(std::size_t n, C c) {
	resize(n);
	if (n)
		T::assign(ptr_, n, c);
	return *this;
}



/** str.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign( const ya_basic_string<C,T,A>& str ) {
	if (this != &str)
		return assign(str.ptr_, str.size());
	return *this;
}


/** cstr .
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign(const C*	cstr) {
	return assign(cstr, T::length(cstr));
}


/** caryn.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign(const C* cary, std::size_t n) {
	resize(n);
	if (n) {
		assert(cary != NULL);
		// if (ptr_ != cary)
		T::copy( ptr_, cary, n);
	}
	return *this;
}


/** strAposڂAnoĐݒ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign( const ya_basic_string<C,T,A>& str, std::size_t	pos, std::size_t n ) {
	str.check_pos_len(pos, n);
	resize(n);
	if (n) {
		T::copy( ptr_, str.ptr(pos), n);
	}
	return *this;
}


/** Ce[^̓͂.
 */
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template<class InpIte>
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign(InpIte first, InpIte last)
#else
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::assign(const C* first, const C* last)
#endif
{
	std::size_t	l = std::distance(first, last);
	resize(l);
	if (l) {
		std::copy(first, last, ptr_);
	}
	return *this;
}



/** ary  posڂ nRs[.
 */
template< typename C, class T, class A >
std::size_t
ya_basic_string<C,T,A>::copy(C* ary, std::size_t n, std::size_t pos) const {
	if (ary == 0) {
		outOfRng();
		return 0;
	}
	if (n == 0)
		return 0;
	check_pos_len(pos, n);
	if (n)
		T::copy(ary, ptr(pos), n);
	return n;
}




/** posڂnAVKĕԂ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>
ya_basic_string<C,T,A>::substr(std::size_t pos, std::size_t n) const {
	check_pos_len(pos, n);
	return ya_basic_string<C,T,A>( ptr(pos), n );
}



// -------------------------------
// r.
// a.compare(b) abƂ̔r. ʂstrcmp()ƓAȂa<bA0Ȃa==b, Ȃa>b\.


/** str2 Ɣr.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(const ya_basic_string<C,T,A>& str2) const {
	std::size_t		l1 = size_;
	std::size_t		l2 = str2.size_;
	difference_type	d  = difference_type(l1 - l2);
	if (d > 0)
		l1 = l2;
	int c = 0;
	if (l1 > 0)
		c = T::compare(ptr_, str2.ptr_, l1);
	if (c == 0)
		c = d;
	return c;
}



/* pos1ڂn1 str2 Ɣr.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(std::size_t	pos1, std::size_t n1, const ya_basic_string<C,T,A>& str2) const
{
	std::size_t	l1 = size_;
	std::size_t	l2 = str2.size_;
	if (pos1 > l1) {
		outOfRng();
		pos1 = l1;
	}
	if (n1 > l1 - pos1)
		n1 = l1 - pos1;
	difference_type	d = difference_type (n1 - l2);
	if (d > 0)
		n1 = l2;
	int c = 0;
	if (n1 > 0)
		c = T::compare(ptr(pos1), str2.ptr_, n1);
	if (c == 0)
		c = d;
	return c;
}



/** pos1ڂn1str2pos2ڂn2Ɣr.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(
		std::size_t						pos1,
		std::size_t						n1,
		const ya_basic_string<C,T,A>& 	str2,
		std::size_t						pos2,
		std::size_t						n2
) const
{
	std::size_t	l1 = size_;
	std::size_t	l2 = str2.size_;
	if (pos1 > l1) {
		outOfRng();
		pos1 = l1;
	}
	if (n1 > l1 - pos1)
		n1 = l1 - pos1;
	if (pos2 > l2) {
		outOfRng();
		pos2 = l2;
	}
	if (n2 > l2 - pos2)
		n2 = l2 - pos2;
	difference_type d = difference_type(n1 - n2);
	if (d > 0)
		n1 = n2;
	int c = 0;
	if (n1 > 0)
		c = T::compare(ptr(pos1), str2.ptr(pos2), n1);
	if (c == 0)
		c = d;
	return c;
}



/** cstr2 Ƃ̔rs.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(const C* cstr2) const
{
	std::size_t 	l1 = size_;
	std::size_t 	l2 = T::length(cstr2);
	difference_type d  = difference_type(l1 - l2);
	if (d > 0)
		l1 = l2;
	int c = 0;
	if (l1 > 0)
		c = T::compare(ptr_, cstr2, l1);
	if (c == 0)
		c = d;
	return c;
}



/** cary2  nƔr.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(const C* cary2, std::size_t n2) const
{
	std::size_t 	n1 = size_;
	difference_type d  = difference_type(n1 - n2);
	if (d > 0)
		n1 = n2;
	int c = 0;
	if (n1 > 0)
		c = T::compare(ptr_, cary2, n1);
	if (c == 0)
		c = d;
	return c;
}



/** pos1ڂn1  cstr2 r.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(std::size_t pos1, std::size_t n1, const C* cstr2) const
{
	std::size_t l1 = size_;
	std::size_t n2 = T::length(cstr2);
	if (pos1 > l1) {
		outOfRng();
		pos1 = l1;
	}
	if (n1 > l1 - pos1)
		n1 = l1 - pos1;
	difference_type d = difference_type(n1 - n2);
	if (d > 0)
		n1 = n2;
	int c = 0;
	if (n1 > 0)
		c = T::compare(ptr(pos1), cstr2, n1);
	if (c == 0)
		c = d;
	return c;
}



/** pos1ڂn1 cary2  n2Ɣr.
 * @return 		,0,(strcmp()ɓ)
 */
template< typename C, class T, class A >
int ya_basic_string<C,T,A>::compare(std::size_t pos1, std::size_t n1, const C* cary2, std::size_t n2) const
{
	std::size_t l1 = size_;
	if (pos1 > l1) {
		outOfRng();
		pos1 = l1;
	}
	if (n1 > l1 - pos1)
		n1 = l1 - pos1;
	difference_type d = difference_type(n1 - n2);
	if (d > 0)
		n1 = n2;
	int c = 0;
	if (n1 > 0)
		c = T::compare(ptr(pos1), cary2, n1);
	if (c == 0)
		c = d;
	return c;
}



// -------------------------------
// ǉ.

/** strǉ.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::operator+=(const ya_basic_string<C,T,A>& str)
{
	return append(str);
}


/** cstr̒ǉ.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::operator+=(const C* cstr)
{
	return append(cstr);
}


/** c̒ǉ.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>& ya_basic_string<C,T,A>::operator+=(C c) {
	if (size_+1 >= capa_) {
		reserve(size_+1);
	}
	ptr_[size_] = c;
	++size_;
	return *this;
}


/** cnǉ.
 */
template< typename C, class T, class A > inline
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append(std::size_t n, C c)
{
	if (n) {
		std::size_t l1 = size_;
		resize(l1 + n);
		T::assign(ptr(l1), n, c);
	}
	return *this;
}


/** cstr̒ǉ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append(const C*	cstr2)
{
	std::size_t l2 = T::length(cstr2);
	if (l2) {
		std::size_t l1 = size_;
		resize(l1 + l2);
		T::copy(ptr(l1), cstr2, l2);
	}
	return *this;
}


/** carynǉ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append(const C*	cary2, std::size_t n2)
{
	if (n2) {
		std::size_t l1 = size_;
		resize(l1 + n2);
		T::copy(ptr(l1), cary2, n2);
	}
	return *this;
}


/** str2ǉ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append( const ya_basic_string<C,T,A>& str2)
{
	std::size_t l2 = str2.size_;
	if (l2) {
		std::size_t l1 = size_;
		resize(l1+l2);
		T::copy(ptr(l1), str2.ptr_ , l2 );
	}
	return *this;
}


/** str2pos2n2ǉ.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append( const ya_basic_string<C,T,A>& str2, std::size_t pos2, std::size_t n2 )
{
	str2.check_pos_len(pos2, n2);
	if (n2) {
		std::size_t l1 = size_;
		resize(l1+n2);
		T::copy( ptr(l1), str2.ptr(pos2), n2 );
	}
	return *this;
}


/** Ce[^Œǉ.
 */
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template<class InpIte>
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append(InpIte first, InpIte last)
#else
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::append(const C* first, const C* last)
#endif
{
	std::size_t	l2 = std::distance(first, last);
	if (l2) {
		std::size_t 	l1 = size_;
		resize(l1+l2);
		std::copy(first, last, ptr(l1));
	}
	return *this;
}



// -------------------------------
// 폜

/** posڂn폜
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::erase(std::size_t pos, std::size_t n)
{
	assert(ptr_ != 0);
	std::size_t	sz = size_;
	C*	ptr= ptr_;
	C*	b  = ptr + pos;
	if (n == npos || pos+n >= sz) {
		if (pos >= sz)
			return *this;	//ptr + sz;
		destruct(b, sz - pos);
		size_ = pos;
		return *this; //b;
	}
	C*	e = b + n;
	for (C* d = b; d != e; ++d) {
		*d = *(d+n);
	}
	destruct(e, n);
	size_ = sz - n;
	return *this;	// e;
}



/** posڂ1폜
 */
template< typename C, class T, class A >
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::erase(typename ya_basic_string<C,T,A>::iterator pos)
{
	C*	b = begin();
	C*	e = end();
	if (pos <  b || e <= pos) {			//  pos=end()̓G[Ȃ.
		assert(b <= pos && pos <= e);
		return e;
	}
	--e;
	for (C* p = pos; p != e; ++p)
		*p = *(p+1);
	A::destroy(e);
	--size_;
	return pos;
}



/** firstlastO܂ł̕폜
 */
template< typename C, class T, class A >
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::erase(
		typename ya_basic_string<C,T,A>::iterator first,
		typename ya_basic_string<C,T,A>::iterator last)
{
	C*	b = begin();
	C*	e = end();
	if (first > last || first < b || last > e) {
		assert(first <= last && b <= first && last <= e);
		return e;
	}
	std::size_t l = last - first;
	if (l) {
		e -= l;
		for (C* p = first; p != e; ++p)
			*p = *(p+l);
		destruct(e, l);
		size_ -= l;
	}
	return first;
}




// -------------------------------
// }

/** c1Apos1ɑ}.
 */
template< typename C, class T, class A >
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::insert(typename ya_basic_string<C,T,A>::iterator pos, C c)
{
	return insert(pos, 1U, c);
}


/** c2n2Apos1ɑ}.
 */
template< typename C, class T, class A >
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::insert(
		typename ya_basic_string<C,T,A>::iterator 	pos,
		std::size_t									n,
		C											c)
{
	if (n == 0)
		return pos;

	C*	b = ptr_;
	if (size_ + n > capa_) {
		difference_type		diff = pos - b;
		reserve( size_ + n );
		b	= ptr_;
		pos = b + diff;
	}
	C*	e = b + size_;
	if (pos < b || pos > e) {
		outOfRng();
		return e;
	}
	size_ += n;
	for (C* p = e; --p >= pos;) {
		A::construct(p+n, *p);
		A::destroy(p);
	}
	e = pos + n;
	while (pos < e)
		A::construct(pos++, c);
	return pos;
}



/** [first,last) ͈͂pos֑}.
 */
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template <typename InpIte>
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::insert(typename ya_basic_string<C,T,A>::iterator pos, InpIte first, InpIte last)
#else
template< typename C, class T, class A >
typename ya_basic_string<C,T,A>::iterator
ya_basic_string<C,T,A>::insert(typename ya_basic_string<C,T,A>::iterator pos, const C* first, const C* last)
#endif
{
	std::size_t n = std::distance(first, last);
	if (n == 0)
		return pos;
	C*	b = ptr_;
	if (size_ + n > capa_) {
		difference_type		diff = pos - b;
		reserve( size_ + n );
		b	= ptr_;
		pos = b + diff;
	}
	C*	e = b + size_;
	if (pos < b || pos > e) {
		outOfRng();
		return e;
	}
	size_ += n;
	for (C* p = e; --p >= pos;) {
		A::construct(p+n, *p);
		A::destroy(p);
	}
	e = pos + n;
	while (pos < e) {
		A::construct(pos++, *first);
		++first;
	}
	return pos;
}



/** str2pos2n2Apos1ɑ}.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::insert(
		std::size_t						pos1,
		const ya_basic_string<C,T,A>&	str2,
		std::size_t						pos2,
		std::size_t						n2	)
{
	str2.check_pos_len(pos2, n2);
	if (n2) {
		check_pos(pos1);
		insert( ite(pos1), str2.ptr(pos2), str2.ptr(pos2+n2) );
	}
	return *this;
}


/** c2n2Apos1ɑ}.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::insert(std::size_t pos1, std::size_t n2, C c2)
{
	if (n2) {
		check_pos(pos1);
		insert(ite(pos1), n2, c2);
	}
	return *this;
}


/** cstr2Apos1ɑ}.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::insert(std::size_t pos1, const C* cstr2)
{
	check_pos(pos1);
	std::size_t l2 = T::length(cstr2);
	if (l2) {
		insert(ite(pos1), &cstr2[0], &cstr2[l2]);
	} else {
		resize(pos1);
	}
	return *this;
}


/** cary2n2Apos1ɑ}.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::insert(std::size_t pos1, const C* cary2, std::size_t n2)
{
	check_pos(pos1);
	if (n2) {
		insert(ite(pos1), &cary2[0], &cary2[n2]);
	} else {
		resize(pos1);
 	}
	return *this;
}



// -------------------------------
// u
#if 1
/** pos1n1Astr2pos2n2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
		std::size_t						pos1,
		std::size_t						n1	,
		const ya_basic_string<C,T,A>&	str2,
		std::size_t						pos2,
		std::size_t						n2	)
{
	str2.check_pos_len(pos2, n2);
	check_pos_len(pos1, n1);
	erase( ite(pos1), ite(pos1+n1) );
	insert( ite(pos1), str2.ite(pos2), str2.ite(pos2+n2) );
	return *this;
}
#endif



/** pos1n1Ac2n2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(std::size_t pos1, std::size_t n1, std::size_t n2, C c2)
{
	check_pos_len(pos1, n1);
	erase( ite(pos1), ite(pos1+n1) );
	insert(ite(pos1), n2, c2);
	return *this;
}


/** pos1n1Acstr2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(std::size_t pos1, std::size_t n1, const C* cstr2)
{
	check_pos_len(pos1, n1);
	erase( ite(pos1), ite(pos1+n1) );
	std::size_t l2 = T::length(cstr2);
	insert(ite(pos1), &cstr2[0], &cstr2[l2]);
	return *this;
}


/** pos1n1Acary2n2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(std::size_t pos1, std::size_t n1, const C* cary2, std::size_t n2)
{
	check_pos_len(pos1, n1);
	erase( ite(pos1), ite(pos1+n1) );
	insert(ite(pos1), &cary2[0], &cary2[n2]);
	return *this;
}




/** [first1,last1)͈̔͂Astr2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	const ya_basic_string<C,T,A>&				str2)
{
	check_range(first1, last1);
	erase(first1, last1);
	insert( first1, str2.begin(), str2.end() );
	return *this;
}



/** [first1,last1)͈̔͂AcstrɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	const C*									cstr2)
{
	check_range(first1, last1);
	erase(first1, last1);
	std::size_t l2 = T::length(cstr2);
	insert( first1, &cstr2[0], &cstr2[l2] );
	return *this;
}



/** [first1,last1)͈̔͂AcarynɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	const C*									cary2,
	std::size_t									n2	)
{
	check_range(first1, last1);
	erase(first1, last1);
	insert( first1, &cary2[0], &cary2[n2] );
	return *this;
}



/** [first1,last1)͈̔͂Ac2n2ɁAu.
 */
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	std::size_t									n2,
	C											c2)
{
	check_range(first1, last1);
	erase(first1, last1);
	insert(first1, n2, c2);
	return *this;
}



/** [first1,last1)͈̔͂A̓Ce[^[first2,last2)̕ɁAu.
 */
#ifndef YA_STRING_NO_MEMBER_TEMPLATES	// o[ev[gT|[g̏ꍇ.
template< typename C, class T, class A >
template< typename InpIte >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	InpIte										first2,
	InpIte										last2)
#else
template< typename C, class T, class A >
ya_basic_string<C,T,A>&
ya_basic_string<C,T,A>::replace(
	typename ya_basic_string<C,T,A>::iterator	first1,
	typename ya_basic_string<C,T,A>::iterator	last1,
	const C*									first2,
	const C*									last2)
#endif
{
	check_range(first1, last1);
	erase(first1,	last1);
	insert(first1, first2, last2);
	return *this;
}



// -------------------------------
// 


/**  c2̌ʒuAnposAԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(C c2) const {
	return find_1(ptr_, 0, size_, c2);
}



/** pos1ȍ~  c2 ̌ʒuAnposAԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(C c2, std::size_t pos1) const {
	std::size_t l1 = size_;
	if (pos1 > l1)
		pos1 = l1;
	l1 -= pos1;
	return find_1(ptr_, pos1, l1, c2);
}


/** str2TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(const ya_basic_string<C,T,A>& str2) const
{
	//x return find(str2.ptr_, 0, str2.size_);
	return find_str( ptr_, 0, size_, str2.ptr_, str2.size_ );
}


/** pos1  str2TAʒuԂ. nposԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return find(str2.ptr_, pos1, str2.size_);
}


/** cstr2 TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(const C* cstr2) const
{
	std::size_t l2 = cstr2 ? T::length(cstr2) : 0;
	//x return find(cstr2, 0, l2);
	return find_str(ptr_, 0, size_, cstr2, l2);
}


/** pos1  cstr2 TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find(const C* cstr2, std::size_t pos1) const
{
	std::size_t l2 = cstr2 ? T::length(cstr2) : 0;
	return find(cstr2, pos1, l2);
}


/** pos1  cary2nTAʒuԂ. nposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	std::size_t		n1 = size_;
	if (pos1 > n1)
		return npos;	// pos1 = n1;
	n1 -= pos1;
	return find_str(ptr_, pos1, n1, cary2, n2);
}



/// pos1  cary2nTAʒuԂ. nposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_str(const C* cary1, std::size_t ofs1, std::size_t n1, const C* cary2, std::size_t n2)
{
	if (ofs1 > n1)
		return npos;	// pos1 = n1;

	// TȂ΁A0 (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return ofs1;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 > n1)	// n2̂قጩ͂Ȃ
		return npos;

	if (n2 == 1)	// 11p
		return find_1(cary1, ofs1, n1, cary2[0]);

	const C*	src1	 = cary1+ofs1;
	const C*	cary1end = src1 + n1 - n2;
	const C*	key2	 = cary2;
	const C*	key2end  = key2 + n2;
	C 			k2		 = *key2++;		// p^[̍ŏ

  RETRY:
	while (src1 <= cary1end) {
		int c = *src1++;
		if (c == k2) {
			const C* s1  = src1;
			const C* kp2 = key2;
			// if (k2 != key2end)		// keyE - key >= 1 Ȃ̂ŁAifsv
			{
				do {
					//x if (t == se) return npos;
					c = *s1++;
					c = c - *kp2++;
					if (c)
						goto RETRY;
				} while (kp2 != key2end);
			}
			return src1 - 1 - cary1;
		}
	}
	return npos;
}


///  c2̌ʒuAnposAԂ
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_1(const C* cary1, std::size_t ofs1, std::size_t n1, C c2)
{
	if (n1 == 0 || cary1 == 0)
		return npos;
	const C* 	p1 = T::find(cary1+ofs1, n1, c2);
	std::size_t n  = npos;
	if (p1)
		n = p1 - cary1;
	return n;
}



//-----------------------
// tT

/** t  c2̌ʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(C c2) const {
	return rfind_1(ptr_, size_, c2);
}



/** t pos1   c2 ̌ʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(C c2, std::size_t pos1) const {
	if (pos1 < npos)
		++pos1;
	std::size_t l1 = size_;
	if (l1 > pos1)
		l1 = pos1;
	return rfind_1(ptr_, l1, c2);
}



/** t str2 TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(const ya_basic_string<C,T,A>& str2) const
{
	return rfind_str( ptr_, size_, str2.ptr_, str2.size_ );
}


/** t pos1  str2TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return rfind(str2.ptr_, pos1, str2.size_);
}


/** t cstr2 TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(const C* cstr2) const
{
	std::size_t l2 = cstr2 ? T::length(cstr2) : 0;
	return rfind_str(ptr_, size_, cstr2, l2);
}


/** t pos1  cstr2 TAʒuԂ. nposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::rfind(const C* cstr2, std::size_t pos1) const
{
	std::size_t l2 = cstr2 ? T::length(cstr2) : 0;
	return rfind(cstr2, pos1, l2);
}


/** t pos1  cary2nTAʒuԂ. nposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::rfind(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	if (pos1 < npos)
		++pos1;
	std::size_t		n1 = size_;
	if (n1 > pos1)
		n1 = pos1;
	return rfind_str(ptr_, n1, cary2, n2);
}


/// t pos1  cary2nTAʒuԂ. nposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::rfind_str(const C* cary1, std::size_t n1, const C* cary2, std::size_t n2)
{
	// TȂ΁A0 (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return 0;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 > n1)	// n2̂قጩ͂Ȃ
		return npos;

	if (n2 == 1)	// 11p
		return rfind_1(cary1, n1, cary2[0]);

	//const C*	cary1end = cary1 + n1 - n2;
	const C*	cary1end = cary1 + n1; // - n2;
	const C*	src1	 = cary1end;
	const C*	key2	 = cary2;
	const C*	key2end  = key2 + n2;
	C 			k2		 = *key2++;		// p^[̍ŏ

  RETRY:
	while (--src1 >= cary1) {
		int c = *src1;
		if (c == k2) {
			const C* s1  = src1+1;
			const C* kp2 = key2;
			// if (k2 != key2end)		// keyE - key >= 1 Ȃ̂ŁAifsv
			{
				do {
					//x if (t == se) return npos;
					c = *s1++;
					c = c - *kp2++;
					if (c)
						goto RETRY;
				} while (kp2 != key2end);
			}
			return src1 - cary1;
		}
	}
	return npos;
}



/// tT cary1n1  c2΂̈ʒuAnposAԂ.
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::rfind_1(const C* cary1, std::size_t n1, C c2)
{
	if (n1 == 0 || cary1 == 0)
		return npos;
	const C* 	s1 = cary1 + n1;
	do {
		--s1;
		if (*s1 == c2)
			return s1 - cary1;
	} while (s1 >= cary1);
	return npos;
}



//------------------------
// w蕶񒆂̂Âꂩꕶƃ}b`̂T.


/**  čʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_of(C c) const {
	return find(c);
}


/** pos1ȍ~  c2 ̌ʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_of(C c2, std::size_t pos1) const {
	return find(c2, pos1);
}



/** pos1ڂn1̒ɁAcary2n2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_of(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	std::size_t		n1 = size_;
	if (pos1 > n1)
		return npos;	// pos1 = n1;
	n1 -= pos1;
	return fnd_first_of(ptr_, pos1, n1, cary2, n2);
}



/** pos1ڂn1̒ɁAstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_of(const ya_basic_string<C,T,A>& str2) const
{
	return fnd_first_of(ptr_, 0U, size_, str2.ptr_, str2.size_);
}



/** pos1ڂn1̒ɁAstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_of(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return find_first_of(str2.ptr_, pos1, str2.size_);
}




/** pos1ڂn1̒ɁAcstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_first_of(const C* cstr2) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	std::size_t		n1 = size_;
	return fnd_first_of(ptr_, 0, n1, cstr2, n2);
}



/** pos1ڂn1̒ɁAcstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_first_of(const C* cstr2, std::size_t pos1) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	return find_first_of( cstr2, pos1, n2 );
}



/// cary1ofs1ڂn1̒n2cary2̉ꂩ̕邩T.
/// ΂̈ʒuAȂnposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t
ya_basic_string<C,T,A>::fnd_first_of(
		const C*	cary1,
		std::size_t ofs1,
		std::size_t n1,
		const C*	cary2,
		std::size_t n2)
{
	// TȂ΁Anpos (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return npos;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 == 1)	// 11p
		return find_1(cary1, ofs1, n1, cary2[0]);

	const C*	s1 = cary1+ofs1;
	do {
		char_type c1 = *s1++;
		const C* p = T::find(cary2, n2, c1);
		if (p)	// A
			return	s1-1-cary1;
	} while (--n1);
	return npos;
}



//------------------------
// w蕶񒆂̂Âꂩꕶƃ}b`̂tT.


/**  čʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_of(C c) const {
	return rfind(c);
}


/** pos1ȍ~  c2 ̌ʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_of(C c2, std::size_t pos1) const {
	return rfind(c2, pos1);
}


/** pos1ڂn1̒ɁAcary2n2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_of(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	if (pos1 < npos)
		++pos1;
	std::size_t		n1 = size_;
	if (n1 > pos1)
		n1 = pos1;
	//n1 -= pos1;
	return fnd_last_of(ptr_, n1, cary2, n2);
}


/** pos1ڂn1̒ɁAstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_of(const ya_basic_string<C,T,A>& str2) const
{
	return fnd_last_of(ptr_, size_, str2.ptr_, str2.size_);
}


/** pos1ڂn1̒ɁAstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_of(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return find_last_of(str2.ptr_, pos1, str2.size_);
}



/** pos1ڂn1̒ɁAcstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_last_of(const C* cstr2) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	return fnd_last_of(ptr_, size_, cstr2, n2);
}


/** pos1ڂn1̒ɁAcstr2̉ꂩ1邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_last_of(const C* cstr2, std::size_t pos1) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	return find_last_of( cstr2, pos1, n2 );
}


/// cary1ofs1ڂn1̒n2cary2̉ꂩ̕邩T.
/// ΂̈ʒuAȂnposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t
ya_basic_string<C,T,A>::fnd_last_of(
		const C*	cary1,
		std::size_t n1,
		const C*	cary2,
		std::size_t n2)
{
	// TȂ΁Anpos (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return npos;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 == 1)	// 11p
		return rfind_1(cary1, n1, cary2[0]);

	const C*	s1 = cary1+n1;
	do {
		char_type c1 = *--s1;
		const C* p = T::find(cary2, n2, c1);
		if (p)	// A
			return	s1-cary1;
	} while (s1 > cary1);
	return npos;
}




//------------------------
// w蕶񒆂̂Â̈ꕶƂ}b`Ȃ̂T.


/**  cȊǑʒuAnposAԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(C c) const {
	return fnd_first_not_of(ptr_, 0, size_, c);
}


/** pos1ȍ~  c2 ȊǑʒuAnposAԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(C c2, std::size_t pos1) const {
	std::size_t n1 = size_;
	if (pos1 > n1)
		return npos;	//x pos1 = l1;
	n1 -= npos;
	return fnd_first_not_of(ptr_, pos1, n1, c2);
}


/** pos1ڂn1̒ɁAcary2n2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	std::size_t		n1 = size_;
	if (pos1 > n1)
		return npos;	// pos1 = n1;
	n1 -= pos1;
	return fnd_first_not_of(ptr_, pos1, n1, cary2, n2);
}


/** pos1ڂn1̒ɁAstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(const ya_basic_string<C,T,A>& str2) const
{
	return fnd_first_not_of(ptr_, 0U, size_, str2.ptr_, str2.size_);
}


/** pos1ڂn1̒ɁAstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return find_first_not_of(str2.ptr_, pos1, str2.size_);
}


/** pos1ڂn1̒ɁAcstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(const C* cstr2) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	std::size_t		n1 = size_;
	return fnd_first_not_of(ptr_, 0, n1, cstr2, n2);
}


/** pos1ڂn1̒ɁAcstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_first_not_of(const C* cstr2, std::size_t pos1) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	return find_first_not_of( cstr2, pos1, n2 );
}


/// cary1ofs1ڂn1̒n2cary2̉ꂩ̕邩T.
/// ΂̈ʒuAȂnposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t
ya_basic_string<C,T,A>::fnd_first_not_of(
		const C*	cary1,
		std::size_t ofs1,
		std::size_t n1,
		const C*	cary2,
		std::size_t n2)
{
	// TȂ΁Anpos (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return ofs1;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 == 1)	// 11p
		return fnd_first_not_of(cary1, ofs1, n1, cary2[0]);

	const C*	s1 = cary1+ofs1;
	do {
		char_type c1 = *s1++;
		const C* p = T::find(cary2, n2, c1);
		if (p == 0)	// A
			return	s1-1-cary1;
	} while (--n1);
	return npos;
}


///  cȊǑʒuAnposAԂ.
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::fnd_first_not_of(const C* cary1, std::size_t ofs1, std::size_t n1, C c2)
{
	if (cary1 == 0 || n1 == 0)
		return npos;
	const C* s = cary1 + ofs1;
	const C* e = s + n1;
	do {
		if (*s != c2)
			return s - cary1;
		++s;
	} while (s != e);
	return npos;
}



//------------------------
// w蕶񒆂̂Â̈ꕶƂ}b`Ȃ̂tT.


/** tŁA cȊǑʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(C c) const {
	return fnd_last_not_of(ptr_, size_, c);
}


/** tŁApos1ȑO  c2 ȊǑʒuAnposAԂ
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(C c2, std::size_t pos1) const {
	if (pos1 < npos)
		++pos1;
	std::size_t		n1 = size_;
	if (n1 > pos1)
		n1 = pos1;
	return fnd_last_not_of(ptr_, n1, c2);
}


/** tŁApos1ڈȑOɁAcary2n2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(const C* cary2, std::size_t pos1, std::size_t n2) const
{
	if (pos1 < npos)
		++pos1;
	std::size_t		n1 = size_;
	if (n1 > pos1)
		n1 = pos1;
	return fnd_last_not_of(ptr_, n1, cary2, n2);
}


/** tŁAstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(const ya_basic_string<C,T,A>& str2) const
{
	return fnd_last_not_of(ptr_, size_, str2.ptr_, str2.size_);
}


/** tŁAstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A> inline
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(const ya_basic_string<C,T,A>& str2, std::size_t pos1) const
{
	return find_last_not_of(str2.ptr_, pos1, str2.size_);
}


/** tŁAcstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(const C* cstr2) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	std::size_t		n1 = size_;
	return fnd_last_not_of(ptr_, n1, cstr2, n2);
}


/** tŁAcstr2̉1ȊO邩T.
 *	΂̈ʒuAȂnposԂ.
 */
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::find_last_not_of(const C* cstr2, std::size_t pos1) const
{
	std::size_t		n2 = cstr2 ? T::length(cstr2) : 0;
	return find_last_not_of( cstr2, pos1, n2 );
}


/// tŁAcary1n1̒n2cary2̉ꂩ̕邩T.
/// ΂̈ʒuAȂnposԂ.
/// [private]
template<typename C, class T, class A>
std::size_t
ya_basic_string<C,T,A>::fnd_last_not_of(
		const C*	cary1,
		std::size_t n1,
		const C*	cary2,
		std::size_t n2)
{
	// TȂ΁Anpos (ł悢?)
	if (n2 == 0 || cary2 == NULL)
		return n1-1;	//npos;

	if (n1 == 0)	// n1 0Ȃ猩Ȃ͂...
		return npos;

	if (n2 == 1)	// 11p
		return fnd_last_not_of(cary1, n1, cary2[0]);

	const C*		s1 = cary1+n1;
	do {
		char_type	c1 = *--s1;
		const C* 	p  = T::find(cary2, n2, c1);
		if (p == 0)	// Ȃok
			return	s1-cary1;
	} while (--n1);
	return npos;
}


///  cȊǑʒuAnposAԂ
/// [private]
template<typename C, class T, class A>
std::size_t		ya_basic_string<C,T,A>::fnd_last_not_of(const C* cary1, std::size_t n1, C c2)
{
	if (cary1 == 0 || n1 == 0)
		return npos;
	const C* s = cary1 + n1;
	do {
		--s;
		if (*s != c2)
			return s - cary1;
	} while (s != cary1);
	return npos;
}




//---------------------------------------------------------------------------------
//---------------------------------------------------------------------------------
// 񃁃o[֐

/** string + string
 */
template<typename C, class T, class A> inline
ya_basic_string<C,T,A> operator+(const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return ya_basic_string<C,T,A>(lhs) += rhs;
}

/** C* + string
 */
template<typename C, class T, class A> inline
ya_basic_string<C,T,A> operator+(const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return ya_basic_string<C,T,A>(lhs) += rhs;
}

/** C + string
 */
template<typename C, class T, class A> inline
ya_basic_string<C,T,A> operator+(C lhs, const ya_basic_string<C,T,A>& rhs) {
	return ya_basic_string<C,T,A>(1, lhs) += rhs;
}

/** string + C*
 */
template<typename C, class T, class A> inline
ya_basic_string<C,T,A> operator+(const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return ya_basic_string<C,T,A>(lhs) += rhs;
}

/** string + C
 */
template<typename C, class T, class A> inline
ya_basic_string<C,T,A> operator+(const ya_basic_string<C,T,A>& lhs, C rhs) {
	return ya_basic_string<C,T,A>(lhs) += rhs;
}



/** string == string
 */
template<typename C, class T, class A> inline
bool operator==(const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) == 0;
}

/** string == C*
 */
template<typename C, class T, class A> inline
bool operator==(const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) == 0;
}

/** C* == string
 */
template<typename C, class T, class A> inline
bool operator==(const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) == 0;
}



/** string != string
 */
template<typename C, class T, class A> inline
bool operator!=(const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) != 0;
}

/** string != C*
 */
template<typename C, class T, class A> inline
bool operator!=(const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) != 0;
}

/** C* != string
 */
template<typename C, class T, class A> inline
bool operator!=(const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) != 0;
}



/** string < string
 */
template<typename C, class T, class A> inline
bool operator < (const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) < 0;
}

/** string < C*
 */
template<typename C, class T, class A> inline
bool operator < (const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) < 0;
}

/** C* < string
 */
template<typename C, class T, class A> inline
bool operator < (const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) > 0;
}



/** string > string
 */
template<typename C, class T, class A> inline
bool operator > (const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) > 0;
}

/** string > C*
 */
template<typename C, class T, class A> inline
bool operator > (const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) > 0;
}

/** C* > string
 */
template<typename C, class T, class A> inline
bool operator > (const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) < 0;
}


/** string <= string
 */
template<typename C, class T, class A> inline
bool operator <= (const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) <= 0;
}

/** string <= C*
 */
template<typename C, class T, class A> inline
bool operator <= (const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) <= 0;
}

/** C* <= string
 */
template<typename C, class T, class A> inline
bool operator <= (const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) >= 0;
}



/** string >= string
 */
template<typename C, class T, class A> inline
bool operator >= (const ya_basic_string<C,T,A>& lhs, const ya_basic_string<C,T,A>& rhs) {
	return lhs.compare(rhs) >= 0;
}

/** string >= C*
 */
template<typename C, class T, class A> inline
bool operator >= (const ya_basic_string<C,T,A>& lhs, const C* rhs) {
	return lhs.compare(rhs) >= 0;
}

/** C* >= string
 */
template<typename C, class T, class A> inline
bool operator >= (const C* lhs, const ya_basic_string<C,T,A>& rhs) {
	return rhs.compare(lhs) <= 0;
}


#ifndef __WATCOMC__
namespace std {
/** .
 */
template<typename C, class T, class A> inline
void swap(ya_basic_string<C,T,A>& lhs, ya_basic_string<C,T,A>& rhs) {
	lhs.swap(rhs);
}
}
#else
/** .
 */
template<typename C, class T, class A> inline
void swap(ya_basic_string<C,T,A>& lhs, ya_basic_string<C,T,A>& rhs) {
	lhs.swap(rhs);
}
#endif


#if 0	// iostream֌W̓^[QbgŎgȂ̂Ŗ.

/// ostream << ya_basic_string
template<typename C, class T, class A>
basic_ostream<C, T>& operator<<(basic_ostream<C, T>& os, const ya_basic_string<C,T,A>& str) {
	return os << str.c_str();
}

/// istream >> ya_basic_string
template<typename C, class T, class T2, class A>
basic_istream<C,T>& operator>>(basic_istream<C, T>& is, ya_basic_string<C,T2,A>& str);

/// istream̓͂ya_basic_stringŎ󂯎. ftHgŉs܂.
template<class charT, class traits, class Allocator>
std::basic_istream<charT, traits>&
getline(std::basic_istream<charT, traits>& is,
	  	ya_basic_string<charT, traits, Allocator>& str,
	  charT delim = '\n');

#endif



typedef ya_basic_string< char	, ya_char_traits<char	>, ya_allocator<char>	>	ya_string;
typedef ya_basic_string< wchar_t, ya_char_traits<wchar_t>, ya_allocator<wchar_t> >	ya_wstring;



#endif 		// YA_STRING_H
